package org.yaoqiang.bpmn.editor.dialog;

import org.json.JSONObject;
import org.yaoqiang.bpmn.editor.BPMNEditor;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.ConnectLdapPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.ConnectionPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.DeploymentPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.EngineConnectionsPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.LdapConnectionPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.LdapConnectionsPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.LdapEntryPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.LdapPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.MembersPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.OrganizationPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.OrganizationsPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.RetrieveDefinitionsPanel;
import org.yaoqiang.bpmn.editor.dialog.jsonpanels.SearchLdapPanel;
import org.yaoqiang.dialog.Panel;
import org.yaoqiang.dialog.PanelFactory;

/**
 * JSONPanelFactory
 * 
 * @author Shi Yaoqiang(shi_yaoqiang@yahoo.com)
 */
public class JSONPanelFactory extends PanelFactory {

	public JSONPanelFactory(JSONPanelContainer pc) {
		super(pc);
	}

	public JSONPanelContainer getPanelContainer() {
		return (JSONPanelContainer) panelContainer;
	}

	public Panel getPanel(Object obj, String type) {
		if ("deployProcessDefinition".equals(type)) {
			return new DeploymentPanel(panelContainer, (BPMNEditor) obj);
		} else if ("retrieveProcessDefinitionList".equals(type)) {
			return new RetrieveDefinitionsPanel(panelContainer, (BPMNEditor) obj);
		} else if ("engineConnections".equals(type)) {
			return new EngineConnectionsPanel(panelContainer, (BPMNEditor) obj);
		} else if ("connection".equals(type)) {
			return new ConnectionPanel(panelContainer, (JSONObject) obj);
		} else if ("organizations".equals(type)) {
			return new OrganizationsPanel(panelContainer, (BPMNEditor) obj);
		} else if ("organization".equals(type)) {
			return new OrganizationPanel(panelContainer, (JSONObject) obj);
		} else if ("ldapConnections".equals(type)) {
			return new LdapConnectionsPanel(panelContainer, (BPMNEditor) obj);
		} else if ("ldapConnection".equals(type)) {
			return new LdapConnectionPanel(panelContainer, (JSONObject) obj);
		} else if ("ldap".equals(type)) {
			if (obj instanceof BPMNEditor) {
				return new LdapPanel(panelContainer, (BPMNEditor) obj);
			}
		} else if ("ldapEntry".equals(type)) {
			return new LdapEntryPanel(panelContainer, (JSONObject) obj);
		} else if ("searchLdap".equals(type)) {
			return new SearchLdapPanel(panelContainer, (BPMNEditor) obj);
		} else if ("connectLdap".equals(type)) {
			return new ConnectLdapPanel(panelContainer, (BPMNEditor) obj);
		} else if ("selectMembers".equals(type) || "selectRoleOccupants".equals(type)) {
			return new MembersPanel(panelContainer, (JSONObject) obj);
		}

		return null;
	}

}
